@@ -33,11 +33,18 @@
  * you do not delete the provisions above, a recipient may use your
  * version of this file under the terms of either the MPL or the GPL.
  * ***** END LICENSE BLOCK ***** */
+
+/* UI.c */
+
+
+
 #include "floater.h"
 #include "tickler.h"
 #include "comm.h"
 #include "deal.h"
 #include "br.h"
+#include "UI.h"
+#include "UIback.h"
 
 fl_bool myturntoact = FALSE;
 
@@ -49,76 +56,206 @@ extern fl_bool tablehostmode;
 extern char *northname, *southname, *eastname, *westname;
 extern char *curhandID;
 
-#define startshowtricktimer() TclDo("startshowtricktimer")
+static void startshowtricktimer()
+{
+  TclDo("startshowtricktimer");
+}
 
-#define needAuctionUpdate() \
-    (atob(TclGet("needAuctionUpdate")) ? \
-     (TclSet("needAuctionUpdate", "0"), TRUE) : FALSE)
+static int needAuctionUpdate()
+{
+  return (atob(TclGet("needAuctionUpdate")) ?
+	  (TclSet("needAuctionUpdate", "0"), TRUE) : FALSE);
+  
+}
 
-#define TUIclearmatrix() { textrefresh(); TclDo("erasebidplay all"); }
-#define TUIdelayedclearmatrix() { textrefresh(); TclDo("delayedclearmatrix"); }
-#define GUIclearmatrix() TclDo("erasebidplay all")
-#define GUIdelayedclearmatrix() TclDo("after 5000 erasebidplay all")
-#define UIclearmatrix() { if (hasWindows) { GUIclearmatrix(); } \
-  else { TUIclearmatrix(); } }
-#define UIdelayedclearmatrix() { if (hasWindows) { GUIdelayedclearmatrix(); } \
-  else { TUIdelayedclearmatrix(); } }
-#define TUIredrawmatrixcards()
-#define GUIredrawmatrixcards() TclDo("redrawmatrixcards")
-#define UIredrawmatrixcards() { if (hasWindows) { GUIredrawmatrixcards(); } \
-  else { TUIredrawmatrixcards(); } }
 
-#define force_update() \
-	{ if (hasWindows) TclDo("update idletasks"); else textrefresh(); }
+static void UIclearmatrix() {
+  UIbackend->clearmatrix(FALSE);
+}
 
-#define startMyTurnTimer() (hasWindows ? TclDo("startMyTurnTimer") : 0)
-#define stopMyTurnTimer() (hasWindows ? TclDo("stopMyTurnTimer") : 0)
+static void UIdelayedclearmatix() {
+  UIbackend->clearmatrix(TRUE);
+}
 
-#define removecardfromhand(play) TclDo2("removecardfromhand ", (play))
-#define togglepassedcard(play) TclDo3("catch {togglepassedcard ", (play), "}")
+static void UIredrawmatrixcards() {
+  UIbackend->redrawmatrixcards();
+}
 
-#define nodeal() \
-	TclDo("fulldeal {} {} {} {} {} {} {} {} {} {} {} {} {} {} {} {}")
-#define noauction() UIauction(FALSE, FALSE, FALSE)
+static void force_update() {
+  UIbackend->force_update();
+}
+
+static void startMyTurnTimer() {
+  UIbackend->startMyTurnTimer();
+}
+
+static void stopMyTurnTimer() {
+  UIbackend->stopMyTurnTimer();
+}
+
+static void removecardfromhand(const char *play) {
+  UIbackend->removecardfromhand(play);
+}
+
+static void togglepassedcard(const char *play) {
+  UIbackend->togglepassedcard(play);
+}
 
 /* clear the frames/labels showing the auction */
-#define newauction() TclDo("newauction")
+static void newauction() {
+  UIbackend->newauction();
+}
+
+/* change the displayed cards for each hanldeal */
+static void fulldeal(const char *selfs,
+		     const char *selfh,
+		     const char *selfd,
+		     const char *selfc,
+		     
+		     const char *lhos,
+		     const char *lhoh,
+		     const char *lhod,
+		     const char *lhoc,
+			
+		     const char *ps,
+		     const char *ph,
+		     const char *pd,
+		     const char *pc,
+			
+		     const char *rhos,
+		     const char *rhoh,
+		     const char *rhod,
+		     const char *rhoc)
+{
+  UIbackend->fulldeal(selfs, selfh, selfd, selfc,
+		      lhos, lhoh, lhod, lhoc,
+		      ps, ph, pd, pc,
+		      rhos, rhoh, rhod, rhoc);
+}
+
+/* handtosuits - extract the four suits from a hand.
+   The outputs s, h, d, and c should point to sufficient
+   memory to hold at least 13 rank designations plus
+   a terminating '\0' (ie, at least 14 bytes)
+*/
+static void handtosuits(const char *hand,
+			char *s, char *h, char *d, char *c)
+{
+  char *suits[4];
+  const char *cp;
+  int suitno;
+  
+  suits[0] = s;
+  suits[1] = h;
+  suits[2] = d;
+  suits[3] = c;
+  
+
+  cp = hand;
+  for(suitno = 0; suitno < 4; suitno++) {
+    char *sp;
+    sp = suits[suitno];
+    
+    
+    /* scan for beginning of suit */
+    while(*cp != '{')  cp++;
+    cp++;
+
+    /* extract the suit */
+    while(*cp != '}') {
+      *sp = *cp;
+      sp++;
+      cp++;
+    }
+    *sp = '\0';
+
+    /* skip the '}' after the suit */
+    cp++;
+  }
+}
+
+			
+/* same as fulldeal, but with 4 arguments instead of 16 */
+static void fulldeal4(const char *self,
+		      const char *lho,
+		      const char *p,
+		      const char *rho)
+{
+  char selfs[100], selfh[100], selfd[100], selfc[100];
+  char lhos[100], lhoh[100], lhod[100], lhoc[100];
+  char ps[100], ph[100], pd[100], pc[100];
+  char rhos[100], rhoh[100], rhod[100], rhoc[100];
+
+  handtosuits(self, selfs, selfh, selfd, selfc);
+  handtosuits(lho, lhos, lhoh, lhod, lhoc);
+  handtosuits(p, ps, ph, pd, pc);
+  handtosuits(rho, rhos, rhoh, rhod, rhoc);
+
+
+  fulldeal(selfs, selfh, selfd, selfc,
+	   lhos, lhoh, lhod, lhoc,
+	   ps, ph, pd, pc,
+	   rhos, rhoh, rhod, rhoc);
+}
+
+
+static void nodeal() 
+{
+  fulldeal("", "", "", "",
+	   "", "", "", "",
+	   "", "", "", "",
+	   "", "", "", "");
+}
+
+
+
+static void UIauction(fl_bool display,
+		      fl_bool showbuttonbar,
+		      fl_bool hideMatrix);
+
+static void noauction() 
+{
+  UIauction(FALSE, FALSE, FALSE);
+}
+
+
 
 /* Hide the auction after this many cards have been played. */
 #define HIDE_AUCTION 4
 
+
 /* If negative, hide auction after HIDE_AUCTION cards; otherwise hide
    auction this many seconds after the final pass. */
 float auction_hide_time; /* initial value taken from Tcl var of same name */
+
 static fl_bool auction_hide_time_inited = FALSE;
 #define init_auction_hide_time()				\
     ((auction_hide_time = atof(TclGet("auction_hide_time"))),	\
      (auction_hide_time_inited = TRUE))
 
+
+
 void UIwrongpassword(void)
 {
-  status("Wrong password"); /*here*/
+  status("Wrong password");
 }
 
 void UInameinuse(void)
 {
-  status("That name is not available; you are not logged in; please try again"); /*here*/
+  status("That name is not available; you are not logged in; please try again");
 }
 
-/****************************************************************************/
-
-void GUIinsertTalk(char *s)
+void UIinsertTalk(const char *s)
 {
-  assert(hasWindows);
-  TclDo3("global toplevel; $toplevel.cmd.talk insert end {", s, "}");
+  UIbackend->insertTalk(s);
 }
 
-void GUIinsertCmd(char *s)
+void UIinsertCmd(const char *s)
 {
-  assert(hasWindows);
-  TclDo3("global toplevel; $toplevel.cmd.cmdline insert end {", s, "}");
+  UIbackend->insertCmd(s);
 }
 
+
 /*****************************************************************************
 Drawing the auction, the hands, and on and on
 *****************************************************************************/
@@ -146,11 +283,6 @@ semistatic char displayhandpstate[200]; /* state of the play */
 semistatic char displayhandpassstate[100]; /* state of the pass at hearts */
 semistatic int displaypov = -1; /* point of view---what seat is drawn at the bottom */
 
-#if 0
-static int displayhandtrick; /* what trick we're looking at */
-static int displayhandround; /* what round of the auction we're looking at */
-#endif
-
 semistatic fl_bool displaydoubledummy = FALSE;
 semistatic int displayhandscoring;
 
@@ -191,6 +323,19 @@ fl_bool caughtup = TRUE;
 #define westcards(b) ((b) ? (disnumplayers == 3 ? "{} {} {} {2} " : \
 			     handto4str(displaydeck, West)) : "{} {} {} {} ")
 
+
+void setinfoline(const char *s)
+{
+  UIbackend->setinfoline(s);
+}
+
+void setstatusline(const char *s)
+{
+  UIbackend->setstatusline(s);
+}
+
+
+
 /* attempt to kibitz one player -- Syela 040223 */
 void UIkib1(int kibseat)
 {
@@ -259,31 +404,23 @@ void UIspec(void)
 /* what's on the status line? */
 static char *statusline(void)
 {
-  return TclGet("statusline");
+  return (char *) TclGet("statusline");
 }
 
 void UItablehost(void)
 {
   extern int newscoremethod; /* from br.c */
 
-  if (hasWindows) {
-    dTclDo5("menus_tablehost ", tablehostmode ? "1 " : "0 ",
-	    ((newscoremethod >= 0) ?
-	     scoringmethods[newscoremethod].scoringname :
-	     ((scoremethod >= 0) ?
-	      scoringmethods[scoremethod].scoringname :
-	      "x")),
-	    " ", new_competitive ? "Competitive" : "Noncompetitive");
-  }
+  UIbackend->hoststatus(tablehostmode,
+			newscoremethod,
+			new_competitive);
 }
 
-void UIstate(int state)
+void UIcommstate(int state)
 {
-  if (hasWindows)
-    if (state == CONNECTED) {
-      TclDo("activate_table_menus 1");
-      if (tablehostmode) UItablehost();
-    } else TclDo("activate_table_menus 0");
+  UIbackend->commstate(state);
+  if (tablehostmode)
+    UItablehost();
 }
 
 /****************************************************************************
@@ -336,16 +473,27 @@ void UIheartshandover(int ns, int es, int ss)
 }
 /****************************************************************************/
 
-void UIsetstatusline(void)
+/* update the UI's status readouts to reflect the current
+ * network status, hand status, etc. */
+void UIupdatestatus(void)
 {
   extern char *bridgestatus; /* from br.c */
   char connstat[200] = { '\0' };
-#ifdef SERVER
-  static fl_bool inited = FALSE;
-#endif
-
-  /* If we're at a table, the status line and info line will info
-     about the current hand, etc.  Otherwise, do what is below. */
+  struct ui_status status = {
+    displayinghearts,
+    connstat,
+    (const char *) STRDUP(displayhandname),
+    (const char *) STRDUP(statushandvul),
+    (const char *) STRDUP(statushanddlr),
+    (const char *) STRDUP(statuscontract),
+    (const char *) STRDUP(statustolead),
+    (const char *) STRDUP(displaytrickswon),
+    (const char *) STRDUP(statusclaim),
+    (const char *) STRDUP(statusresult),
+    (const char *) STRDUP(statusscore)
+  };
+  
+  
   switch (state) {
   case LIMBO:
     strcpy(connstat, tablehostmode ? "Hosting" : "Not at a table");
@@ -358,27 +506,12 @@ void UIsetstatusline(void)
 	strlen(bridgestatus) == 0) {
 #ifdef LOGINSERVER
       strcpy(connstat, "Login server");
-      if (!inited) {
-	TclDo("catch {wm title . {login server}}");
-	TclDo("catch {wm iconname . {Login server}}");
-	inited = TRUE;
-      }
 #endif
 #ifdef RESULTSERVER
       strcpy(connstat, "Result server");
-      if (!inited) {
-	TclDo("catch {wm title . {result server}}");
-	TclDo("catch {wm iconname . {Result server}}");
-	inited = TRUE;
-      }
 #endif
 #ifdef PSEUDOMAILER
       strcpy(connstat, "Pseudomailer");
-      if (!inited) {
-	TclDo("catch {wm title . {pseudomailer}}");
-	TclDo("catch {wm iconname . {pseudomailer}}");
-	inited = TRUE;
-      }
 #endif
 #ifndef SERVER
       strcpy(connstat, tablehostmode ? "Hosting" : "Connected");
@@ -387,66 +520,11 @@ void UIsetstatusline(void)
   }
 
 #if !SILENT
-  if (!hasWindows) {
-    update_statusbar();
-    draw_on_matrix_screen();
-    TclDo3("connstat {", connstat, "}");
-    TclDo3("displayhandname {", displayhandname, "}");
-    TclDo3("statushandvul {", statushandvul, "}");
-    if (!displayinghearts) {
-      TclDo3("statushanddlr {", statushanddlr, "}");
-      TclDo3("statuscontract {", statuscontract, "}");
-    }
-    TclDo3("statustolead {", statustolead, "}");
-    TclDo3("displaytrickswon {", displaytrickswon, "}");
-    TclDo3("statusclaim {", statusclaim, "}");
-    TclDo3("statusresult {", statusresult, "}");
-    TclDo3("statusscore {", statusscore, "}");
-    move_to_curyx();
-  } else {
-#define appendifexists(s, nl)				\
-    {							\
-      if (strlen(connstat) > 0 && strlen(s) > 0)	\
-	strcat(connstat, nl ? "|" : "; ");				\
-      strcat(connstat, s);				\
-    }
-						      
-    appendifexists(displayhandname, 0);
-    appendifexists(statushandvul, 0);
-    if (!displayinghearts) {
-      appendifexists(statushanddlr, 1);
-      appendifexists(statuscontract, 0);
-    }
-    setstatusline(connstat);
-    
-    connstat[0] = '\0';
-    appendifexists(statusscore, 0);
-    appendifexists(displaytrickswon, 1);
-    {
-      char *temp = display_of_previous_trick();
-
-      /* these nested if's try to pick the one most important thing
-       * to put onto the last part of the infoline */
-      if (strexists(temp)) {
-	   appendifexists(temp, 1);
-      }
-      else {
-	   if(strlen(statusresult) > 0) {
-		appendifexists(statusresult, 1);
-	   }
-	   else {
-		appendifexists(statusclaim, 1);
-	   }
-      }
-    }
-    
-    setinfoline(connstat);
-#undef appendifexists
-  }
+  UIbackend->updatestatus(&status);
 #endif /* !SILENT */
 }
 
-static void snafflesuit(int *hand, int suit, char *dest)
+void snafflesuit(int *hand, int suit, char *dest)
 {
   int i;
 
@@ -466,21 +544,19 @@ static void snafflesuit(int *hand, int suit, char *dest)
   *dest = '\0';
 }
 
-/* convert a hand into something of the form "{AKQJ} {AKQJ} {AKQJ} {4} " */
-static char *handto4str(int *deck, int player)
-{
-  char s[100];
-  int i;
 
-  s[0] = '\0';
-  SUITLOOP(i) {
-    sprintf(s + strlen(s), "{");
-    snafflesuit(deck + cardsperhand * player, i, s + strlen(s));
-    sprintf(s + strlen(s), "} ");
-  }
-  return TEMPCOPY(s);
+void UIpatientcursor() 
+{
+  UIbackend->patientcursor();
 }
 
+void UInormalcursor() 
+{
+  UIbackend->normalcursor();
+}
+
+
+
 /* Updates (global) displayhandpassstate and returns whether redisplay
    is necessary (i.e., whether the hands have been changed by the pass). */
 static fl_bool UIupdatepass(char *passstate, int pov)
@@ -509,14 +585,23 @@ static fl_bool UIupdatepass(char *passstate, int pov)
 }
 
 /* dd is whether to show all four hands */
-/* cards in played are not shown */
 /* pass is the hearts pass, if applicable */
 static void UIshowdeal(int pov, fl_bool dd, int dummycol,
-		       char *played, char *pass)
+		       char *pass)
 {
   fl_bool ed, wd, sd, nd;
+  char *played;   /* cards in played are not shown */
 
-  if (hasWindows) UIpatientcursor();
+  
+
+  if(displayhandisover)
+       played = NULL;
+  else
+       played = displayhandpstate;
+
+  
+
+  UIpatientcursor();
   if (displayinghearts || seated || kibitzing1 || dd || dummycol >= 0)
     if (displaydeck == NULL && strlen(displayhandID) > 0) {
       displaydeck = (int *) salloc(sizeof(int) * 52);
@@ -544,23 +629,23 @@ static void UIshowdeal(int pov, fl_bool dd, int dummycol,
     switch (pov) {
     case South:
       if (displayinghearts && seated && myseat == West) 
-	TclDo5("fulldeal ", southcards(sd),
-	       westcards(TRUE), northcards(nd), eastcards(ed));
+	fulldeal4(southcards(sd),
+		  westcards(TRUE), northcards(nd), eastcards(ed));
       else 
-	TclDo5("fulldeal ", southcards(sd || seated || kibitzing1),
-	       westcards(wd), northcards(nd), eastcards(ed));
+	fulldeal4(southcards(sd || seated || kibitzing1),
+		  westcards(wd), northcards(nd), eastcards(ed));
       break;
     case North:
-      TclDo5("fulldeal ", northcards(nd || seated || kibitzing1), eastcards(ed),
-	     southcards(sd), westcards(wd));
+      fulldeal4(northcards(nd || seated || kibitzing1), eastcards(ed),
+		southcards(sd), westcards(wd));
       break;
     case East:
-      TclDo5("fulldeal ", eastcards(ed || seated || kibitzing1), southcards(sd),
-	     westcards(wd), northcards(nd));
+      fulldeal4(eastcards(ed || seated || kibitzing1), southcards(sd),
+		westcards(wd), northcards(nd));
       break;
     case West:
-      TclDo5("fulldeal ", westcards(wd || seated || kibitzing1), northcards(nd),
-	     eastcards(ed), southcards(sd));
+      fulldeal4(westcards(wd || seated || kibitzing1), northcards(nd),
+		eastcards(ed), southcards(sd));
       break;
     default:
       assert(0);
@@ -572,7 +657,7 @@ static void UIshowdeal(int pov, fl_bool dd, int dummycol,
   if (played != NULL) {
     int i;
     char s[4];
-    
+
     for (i = strlen(played) - 2; i >= 0; i -= 2) {
       sprintf(s, "%c %c", played[i], played[i+1]);
       removecardfromhand(s);
@@ -581,7 +666,7 @@ static void UIshowdeal(int pov, fl_bool dd, int dummycol,
   }
 done:
   if (dd) force_update();
-  if (hasWindows) UInormalcursor();
+  UInormalcursor();
 }
 
 #define col_to_name(col, pov) outputname(col_to_fullname((col), (pov)))
@@ -656,43 +741,27 @@ static void UIupdatepov(int pov, fl_bool dd, int dummycol)
   displaydoubledummy = dd;
   displaypov = pov;
 
-  UIshowdeal(pov, dd, dummycol, displayhandpstate, displayhandpassstate);
+  UIshowdeal(pov, dd, dummycol, displayhandpassstate);
   UIshowplayernames(pov);
 }
 
-static void showmatrixbid(int where, char *bid)
-{
-  {
-    int i = 0;
-    
-    while (isspace(bid[i])) i++;
-    assert(isin(bid[i], "12345670?"));
-    i++;
-    while (isspace(bid[i])) i++;
-    assert(isin(tolower(bid[i]), "xprcdhsn?"));
-  }
 
-  switch (where) {
-  case 0:
-    TclDo2("showbid lho ", bid);
-    break;
-  case 1:
-    TclDo2("showbid pard ", bid);
-    break;
-  case 2:
-    TclDo2("showbid rho ", bid);
-    break;
-  case 3:
-    TclDo2("showbid self ", bid);
-    break;
-  default:
-    assert(0);
-  }
+/*  not used, but should be documented somewhere before this is deleted
+void checkbid(char *bid)
+{
+  int i = 0;
+  
+  while (isspace(bid[i])) i++;
+  assert(isin(bid[i], "12345670?"));
+  i++;
+  while (isspace(bid[i])) i++;
+  assert(isin(tolower(bid[i]), "xprcdhsn?"));
 }
+*/
 
 static void showmatrixplay(int where, char *play, int pov)
 {
-  if (!isin('?', play)) removecardfromhand(play);
+/*  if (!isin('?', play)) removecardfromhand(play); */
     
   {
     int i = 0;
@@ -741,8 +810,11 @@ static void showmatrixplay(int where, char *play, int pov)
   }
 }
 
-/* first arg is whether to show the auction at all; second is whether to
-   include the buttonbar */
+/* first arg is whether to show the auction at all;
+   second is whether to include the buttonbar;
+   third is whether it is necessary to display the playing-card area
+*/
+/* XXX  uggh, all this matrix and auction showing and hiding should be moved to UIback-tk.c */
 static void UIauction(fl_bool display, fl_bool showbuttonbar, fl_bool hideMatrix)
 {
   static fl_bool displayed = FALSE;
@@ -775,12 +847,8 @@ static void UIauction(fl_bool display, fl_bool showbuttonbar, fl_bool hideMatrix
   if (display == displayed && showbuttonbar == displayshowbuttonbar
       && hideMatrix == matrixHidden) return;
 
-  if (hasWindows) UIpatientcursor();
+  UIpatientcursor();
 
-  /* The below had been protected by "if (displayed != display)" but a
-     mystery bug with the auction not getting packed on the GUI popped
-     up, and the cost of a redundant showauction is low on GUI.  (On
-     TUI, redundant showauction erases the auction, oddly enough.) */
   if (hasWindows || (displayed != display))
     if ((displayed = display)) 
       TclDo("showauction 1");
@@ -805,22 +873,22 @@ static void UIauction(fl_bool display, fl_bool showbuttonbar, fl_bool hideMatrix
 }
 
 
-static char *auctionframe(char *frame, int col, int line)
+static void UIshowbid(int col, int row,
+		      const char *bidlevel, const char *biddenom)
 {
-  if (!hasWindows) {
-    sprintf(frame, "%d %d", col, line);
-  } else {
-    char cmd[1000];
-    sprintf(cmd, "auction_cell %d %d", col, line);
-    strcpy(frame, TclDo(cmd));
-  }
-  return frame;
+  UIbackend->showbid(col, row, bidlevel, biddenom);
 }
 
+static void UIshowplayerbid(int column,
+			    const char *bidlevel, const char *biddenom)
+{
+  UIbackend->showplayerbid(column, bidlevel, biddenom);
+}
+
+
 static void UIauctionstartup(int pov)
 {
   int i;
-  char frame[100], cmd[200];
 
   if (dealercolumn < 0) {
     dealercolumn = bseat_to_col(DEALNO_TO_DEALER(atoi(displayhandID)), pov);
@@ -832,11 +900,8 @@ static void UIauctionstartup(int pov)
   if (displaycalls == 0) {
     newauction();
     UIclearmatrix();
-    for (i=0; i<dealercolumn; i++) { 
-      auctionframe(frame, i, 0);
-      sprintf(cmd, "drawbid %s - -", frame);
-      TclDo(cmd);
-    }
+    for (i=0; i<dealercolumn; i++)
+      UIshowbid(i, 0, "-", "-");
   }
 }
 
@@ -844,7 +909,6 @@ static void UIauctionstartup(int pov)
 static void UIdisplaywhosecall(int pov)
 {
   int line, column;
-  char frame[100];
 
   if (auctionisover(displayhandastate)) {
     sleep(125);
@@ -853,17 +917,18 @@ static void UIdisplaywhosecall(int pov)
   UIauctionstartup(pov);
   line = (displaycalls + dealercolumn) / 4;
   column = (displaycalls + dealercolumn) % 4;
-  auctionframe(frame, column, line);
-  TclDo3("drawbid ", frame, " ? ?");
-  showmatrixbid(column, "? ?");
+  UIshowbid(column, line, "?", "?");
+  UIshowplayerbid(column, "?", "?");
 }
 
 /* the global displaycalls is the call to display */
 static void UIdisplaycall(char *auction, int pov)
 {
   int call = displaycalls;
-  char cmd[100], frame[50];
+  char cmd[100];
   int line, column;
+  char bidlevel[5], biddenom[5];
+  
 
 #if DBGu
   {
@@ -879,26 +944,27 @@ static void UIdisplaycall(char *auction, int pov)
 
   line = (call + dealercolumn) / 4;
   column = (call + dealercolumn) % 4;
-  auctionframe(frame, column, line);
+
   switch (auction[call * 2 + 1]) {
   case 'r':
-    sprintf(cmd, "drawbid %s 0 xx", frame);
+    strcpy(bidlevel, "0");
+    strcpy(biddenom, "xx");
     break;
   case 'p':
   case 'x':
-    sprintf(cmd, "drawbid %s 0 %c", frame, auction[call * 2 + 1]);
+    strcpy(bidlevel, "0");
+    sprintf(biddenom, "%c", auction[call * 2 + 1]);
     break;
   default:
     assert(isdigit(auction[call * 2]));
-    sprintf(cmd, "drawbid %s %c %c", frame, auction[call * 2],
-	    auction[call * 2 + 1]);
+    sprintf(bidlevel, "%c", auction[call * 2]);
+    sprintf(biddenom, "%c", auction[call * 2 + 1]);
     break;
   }
-  showmatrixbid(column, cmd + strlen(cmd) - 4); 
-  TclDo(cmd);
-#if DBGu
-  DEBUG(status(cmd));
-#endif
+    
+    
+  UIshowbid(column, line, bidlevel, biddenom);
+  UIshowplayerbid(column, bidlevel, biddenom);
 
   displaycalls++;
 }
@@ -928,7 +994,7 @@ static void UIdisplaywhoseplay(char *auction, char *pstate, char *passstate,
     int np = disnumplayers;
     column = hseat_to_col(((displayplays % np) + hcol_to_seat(leadercolumn, pov)) % np, pov);
   }
-  showmatrixplay(column, "? ?", pov);
+  showmatrixplay(column, "? ?", pov);   /* XXX should be factored out */
 }
 
 /* Display dummy, except if I am dummy (or I am kibitzing dummy) then
@@ -939,7 +1005,7 @@ static void UIdisplaydummy(int pov)
                      (declarercolumn ^ 2));
 
   if (displaydoubledummy) return;
-  UIshowdeal(pov, FALSE, dummycol, displayhandpstate, NULL);
+  UIshowdeal(pov, FALSE, dummycol, NULL);
   UIshowplayernames(pov);
 }
 
@@ -970,6 +1036,7 @@ static void UIdisplayplay(char *pstate, int pov)
     }
     sprintf(card, "%c %c", pstate[play*2], pstate[play*2+1]);
     showmatrixplay(column, card, pov);
+    removecardfromhand(card);
   }
   displayplays++;
 }
@@ -993,7 +1060,10 @@ static void UIupdateauction(char *auction, char *pstate, int pov, fl_bool seated
 
   if (auctionisover(auction)) {
     UIsummarizeauction(auction, pov);
-    UIauction((numcards(pstate) < HIDE_AUCTION), FALSE, (numcards(pstate) == 0));
+    UIauction((numcards(pstate) < HIDE_AUCTION),
+	      FALSE,
+	      FALSE);
+    
     if (!change) return;
     calls = numcalls(auction);
     while (displaycalls < calls) UIdisplaycall(auction, pov);
@@ -1010,13 +1080,13 @@ static void UIupdateauction(char *auction, char *pstate, int pov, fl_bool seated
 	TclDo(s);
       }
     }
-    return;
+  } else {
+    UIauction(TRUE, seated, TRUE);
+    calls = numcalls(auction);
+    while (displaycalls < calls) UIdisplaycall(auction, pov);
+    UIdisplaywhosecall(pov);
   }
-    
-  UIauction(TRUE, seated, TRUE);
-  calls = numcalls(auction);
-  while (displaycalls < calls) UIdisplaycall(auction, pov);
-  UIdisplaywhosecall(pov);
+
 }
 
 static void UIupdatetrickswon(char *pstate, int pov)
@@ -1048,6 +1118,13 @@ static void UIupdateplay(char *auction, char *pstate, char *passstate, int pov)
   static char *lastdisplayhandID = NULL;
   static int lastpov = -99;
 
+  if(displayhandisover) {
+    /* don't display any plays, between hands */
+    UIclearmatrix();
+    return;
+  }
+
+  
   /* note the addition of !passisover to the if statement below.  hearts*/
   STRCPY(displayhandpstate, pstate);
   if (strlen(pstate) == 0) 
@@ -1102,6 +1179,12 @@ static void UIupdateplay(char *auction, char *pstate, char *passstate, int pov)
   UIdisplaywhoseplay(auction, pstate, passstate, pov);
 }
 
+static int beepatmyturn() {
+  return atob(TclGet("beepAtMyTurn_"));
+}
+
+
+
 /* the workhorse */
 void UIupdate(void)
 {
@@ -1307,7 +1390,8 @@ void UIupdate(void)
       }
 
       /* display all 52 cards to everyone */
-      UIshowdeal(pov, TRUE, -1, NULL, displayhandpassstate);
+      UIclearmatrix();
+      UIshowdeal(pov, TRUE, -1, displayhandpassstate);
       UIshowplayernames(pov);
       /* UIdelayedclearmatrix(); */
     }
@@ -1380,3 +1464,11 @@ void UIsit(char *name, char ch, int myseat)
   }
   if (hasWindows) UInormalcursor();
 }
+
+
+
+
+void UIbeep(void){
+  UIbackend->beep();
+}
+
