@@ -19,6 +19,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *    Lex Spoon <lex@cc.gatech.edu>
  *
  * Alternatively, the contents of this file may be used under the
  * terms of either the GNU General Public License Version 2 or later
@@ -42,13 +43,13 @@ extern char *vulnames[];
 #define fulltable()
 #define notfulltable()
 
-void UIstate(int state);
+void UIcommstate(int state);
 void UIwrongpassword(void);
 void UInameinuse(void);
 void UIupdate(void);
 void UIplayergone(int which);
 void UIsit(char *name, char ch, int myseat);
-void UIsetstatusline(void);
+void UIupdatestatus(void);
 void UIspec(void);
 void UItablehost(void);
 void UIheartshandover(int ns, int es, int ss);
@@ -60,6 +61,12 @@ void setinfoline(const char *s);
 
 void UIbeep(void);
 
+void UIinsertTalk(const char *s);
+void UIinsertCmd(const char *s);
+
+void UIpatientcursor(void);
+void UInormalcursor(void);
+
 
 #endif /* __ui_H__ */
 
