@@ -47,39 +47,13 @@
 #include "br.h"
 #include "deal.h"
 #include "calendar.h"
+#include "UI.h"
 
 
-/*
-   Basic idea: A tree of tables (or rooms, if you prefer that
-   metaphor).  Each node in the table tree is also the root of a tree
-   of people (kibitzers and players) at that table.  When someone says
-   something, normally it is broadcast to everyone at the table.  To
-   join either sort of tree, you send a message to anyone in the tree
-   requesting to join.  You then get a message back telling you
-   whether it is OK (if not, you may be "shunted"---told an alternate
-   place to try to join).
-   
-   When someone joins a table, that fact is broadcast to the table.
-   When a new table starts, that fact is broadcast to everyone at all tables.
-   
-   If you are hosting a table and you get unexpectedly disconnected
-   from your parent, you attempt to rejoin the table tree.  Being
-   connected to the table tree is considered non-critical---if you
-   miss a few messages, no big deal.  Play at your table may continue
-   while you (in the background) try to rejoin the table tree.  Being
-   connected to the kibitzer tree is critical, so attempting to have
-   an automatic rejoin mechanism is messy (it was tried, and worked at
-   one time, but the new way seems cleaner).  Therefore, the kibitzer
-   tree is one-level only, i.e., everyone is connected to the table
-   host.
+/* see documentation/internals/network.txt for information on the design */
 
-   [Old design:  If you get unexpectedly disconnected from your parent, you
-   automatically attempt to reconnect to the same place.  If that
-   fails, you try connecting to the root of the table tree (if you
-   were a table host) or the root of the kibitzer tree (if you were a
-   kibitzer at a table).]
 
-   This code is not very specific to bridge.  Anything relating to lho
+/* This code is not very specific to bridge.  Anything relating to lho
    or rho or a compass direction is bridge related, and a few things
    are bracketed by comments saying "bridge specific."  The rest is
    generic.  */
@@ -109,14 +83,6 @@ semistatic char tablerootname[FLOATER_MAXNAME+1] = {'?', '\0'};
 semistatic char tablerootaddr[MAXADDRLEN+1] = {'\0'};
 semistatic char tablerootport[MAXPORTLEN+1] = {'\0'};
 
-char loginservername[FLOATER_MAXNAME+1] = {'\0'};
-char loginserveraddr[MAXADDRLEN+1] = {'\0'};
-char loginserverport[MAXPORTLEN+1] = {'\0'};
-
-char resultservername[FLOATER_MAXNAME+1] = {'\0'};
-char resultserveraddr[MAXADDRLEN+1] = {'\0'};
-char resultserverport[MAXPORTLEN+1] = {'\0'};
-
 #ifndef SERVER
 char location[200] = {'\0'};
 #endif
@@ -205,6 +171,7 @@ static int opennormalport(void);
   atoi(TclDo3("global nameinuse; info exists {nameinuse(", upcase(name), ")}"))
 
 
+
 char *testConn2(char *clientversion, char *IPaddr, char *port, char *s)
 {
   if (streq(clientversion, "8") || streq(clientversion, "8x"))
@@ -263,6 +230,54 @@ void savenewemail(char *name, char *x)
 }
 #endif /* LOGINSERVER */
 
+
+
+/*****************************************************************************
+Settings for server addresses.  They are stored in Tcl variables.
+*****************************************************************************/
+
+char *loginservername()
+{
+  return STRDUP(TclGet("loginservername"));
+}
+
+char *loginserveraddr()
+{
+    return STRDUP(TclGet("loginserveraddr"));
+}
+
+char *loginserverport()
+{
+  return STRDUP(TclGet("loginserverport"));
+}
+
+
+char *resultservername()
+{
+  return STRDUP(TclGet("resultservername"));
+}
+
+char *resultserveraddr()
+{
+    return STRDUP(TclGet("resultserveraddr"));
+}
+
+char *resultserverport()
+{
+  return STRDUP(TclGet("resultserverport"));
+}
+
+void setserver(const char *name) 
+{
+  TclSet("loginserveraddr", name);
+  TclSet("resultserveraddr", name);
+  comm_reset();
+}
+
+
+
+
+
 /*****************************************************************************
 Code for keeping track of all the tables out there
 *****************************************************************************/
@@ -690,8 +705,7 @@ childlist *findkchild(char *nom)
   }
   if (loser) return(loser);
 
-  /* look for exact matches, but accounting for ! in the name
-     (???whatever that means! -lex) */
+  /* look for exact matches on the name part only */
   for (l = kchildren; l != NULL; l = l->next) {
     if (strlen(l->c->name) > len && l->c->name[len]=='!' &&
         !strncasecmp(nom, l->c->name, len)) {
@@ -1114,7 +1128,7 @@ int FloaterClose(ClientData clientData, Tcl_Interp *interp,
 		 int argc, const char *argv[])
 {
   char *sock;
-  char *connstr = argv[1];
+  char *connstr = (char *) argv[1];
 
   assert(argc == 2); /* argv[0] is "FloaterClose"; argv[1] is the connstr */
 
@@ -1177,7 +1191,7 @@ int FloaterClose(ClientData clientData, Tcl_Interp *interp,
     }
   }
   if (state == LIMBO) tablehostmode = FALSE;
-  if (hasWindows) UIstate(state);
+  UIcommstate(state);
 #endif
   if_not_connected_reset_tablehostname();
   return TCL_OK;
@@ -1208,7 +1222,7 @@ int receivemessage(ClientData clientData, Tcl_Interp *interp,
 #else
 
   /* parse the message; if empty, ignore it and return */
-  if ((m = parsemessage(argv[1])) == NULL) return TCL_OK;
+  if ((m = parsemessage((char *) argv[1])) == NULL) return TCL_OK;
 
   executionlog("receive: ", argv[1]); 
 
@@ -1249,7 +1263,7 @@ int receivemessage(ClientData clientData, Tcl_Interp *interp,
 #endif
 
   /* take action depending on what the message is */
-  handlemessage(m, argv[1]);
+  handlemessage(m, STRDUP(argv[1]));
   return TCL_OK;
 #endif
 }
@@ -1355,27 +1369,9 @@ static void rebroadcast(message *m)
 
 void commsetup()
 {
-  if (strlen(loginserveraddr) == 0) {
-    /* be careful, as these variables may have been set to (lengthy) garbage */
-       /* XXX so dynamically allocate the space... */
-    strncpy3(loginserveraddr, TclDo("gset loginserveraddr"));
-    strncpy3(loginserverport, TclDo("gset loginserverport"));
-    strncpy3(loginservername, TclDo("gset loginservername"));
-    loginserveraddr[sizeof(loginserveraddr) - 1] = '\0';
-    loginservername[sizeof(loginservername) - 1] = '\0';
-    loginserverport[sizeof(loginserverport) - 1] = '\0';
-  }
-
-  if (strlen(resultserveraddr) == 0) {
-    /* be careful, as these variables may have been set to (lengthy) garbage */
-    strncpy3(resultserveraddr, TclDo("gset resultserveraddr"));
-    strncpy3(resultserverport, TclDo("gset resultserverport"));
-    strncpy3(resultservername, TclDo("gset resultservername"));
-    resultserveraddr[sizeof(resultserveraddr) - 1] = '\0';
-    resultservername[sizeof(resultservername) - 1] = '\0';
-    resultserverport[sizeof(resultserverport) - 1] = '\0';
-  }
-
+  TclSet("loginserveraddr", DEFAULT_SERVER);
+  TclSet("resultserveraddr", DEFAULT_SERVER);
+  
   if (localIPaddr == NULL) {
     Floater_hostname = STRDUP(TclDo("info hostname"));
     localIPaddr = STRDUP(TclGet("localIPaddr"));
@@ -1403,6 +1399,17 @@ static int opennormalport()
   return ((port < 0) ? openport(NULL) : port);
 }
 
+
+static void resetmyname()
+{
+    char newname[100];
+#if 0
+    sprintf(newname, "Doe %s", localport); /* for now, use port # as name */
+#endif
+    sprintf(newname, "No Name");
+    setmyname(newname, FALSE);
+}
+
 /* returns port opened on success, or -1 on failure */
 /* if port is NULL, use any old port */
 static int openport(char *port) 
@@ -1423,12 +1430,7 @@ static int openport(char *port)
       -1 : atoi(localport = STRDUP(TclDo("set openport")));
 
   if (localport != NULL && strlen(myname) == 0) {
-    char newname[100];
-#if 0
-    sprintf(newname, "Doe %s", localport); /* for now, use port # as name */
-#endif
-    sprintf(newname, "No Name");
-    setmyname(newname, FALSE);
+    resetmyname();
   }
 
   return ret;
@@ -1531,7 +1533,7 @@ static void disconnect_all(void)
   /* neither send nor expect iamalive messages */
   TclSet("sendiamalivelist", "");
   TclSet("receiveiamalivelist", "");
-  if (hasWindows) UIstate(state);
+  UIcommstate(state);
   if_not_connected_reset_tablehostname();
 }
 
@@ -1547,6 +1549,13 @@ void sever_all_communication(void)
   state = LIMBO;
 }
 
+void comm_reset()
+{
+  sever_all_communication();
+  resetmyname();
+}
+
+
 
 #ifdef LOGINSERVER
 void loginserver(char *p)
@@ -1673,23 +1682,23 @@ void tablehost(char *addr, char *p)
       addr = tablerootaddr;
       p = tablerootport;
     } else {
-      addr = loginserveraddr;
-      p = loginserverport;
+      addr = loginserveraddr();
+      p = loginserverport();
     }
   }
   if (addr != NULL && p != NULL)
     if (!join(addr, p, TABLE_SERVERs, 1) &&
-	(!streq(p, loginserverport) || !streq(addr, loginserveraddr))) {
+	(!streq(p, loginserverport()) || !streq(addr, loginserveraddr()))) {
       /* didn't work; try loginserver */
-      addr = loginserveraddr;
-      p = loginserverport;
+      addr = loginserveraddr();
+      p = loginserverport();
       if (addr != NULL && p != NULL)
 	join(addr, p, TABLE_SERVERs, 1);
     }
 
   tablehostmode = TRUE;
   addwho(myname);
-  if (hasWindows) UIstate(state);
+  UIcommstate(state);
 }
 #endif /* ifndef SERVER */
   
@@ -1882,7 +1891,7 @@ int join(char *addr, char *port, char *capacity, int numtries)
 	      makemsg5(REQUEST_JOIN, FLOATER_VERSION,
 		       myname, localIPaddr, itoa(lport), capacity));
   state = CONNECTED;
-  if (hasWindows) UIstate(state);
+  UIcommstate(state);
   UInormalcursor();
   return 1;
 }
@@ -1972,8 +1981,8 @@ static char *connect_to_loginserver(void)
   int t, retry;
   char *addr, *port;
 
-  addr = loginserveraddr;
-  port = loginserverport;
+  addr = loginserveraddr();
+  port = loginserverport();
 
   if (isin('!', addr)) {
     char *p, *q;
@@ -2027,8 +2036,8 @@ static char *connect_to_resultserver(void)
   int t, retry;
   char *addr, *port;
 
-  addr = resultserveraddr;
-  port = resultserverport;
+  addr = resultserveraddr();
+  port = resultserverport();
 
   if (isin('!', addr)) {
     char *p, *q;
@@ -2380,15 +2389,15 @@ static void announce_tables(void)
   checknewweek();
 
 /*  sendmessage(connstr, makemsg0(FORGET_TABLES));*/
-  if (strlen(loginserveraddr) > 1)
+  if (strlen(loginserveraddr()) > 0)
     sendmessage(connstr,
-		makemsg4(ANNOUNCE_LOGINSERVER, myname, loginservername,
-			 loginserveraddr, loginserverport));
-  if (strlen(tablerootaddr) > 1)
+		makemsg4(ANNOUNCE_LOGINSERVER, myname, loginservername(),
+			 loginserveraddr(), loginserverport()));
+  if (strlen(tablerootaddr) > 0)
     sendmessage(connstr, makemsg4(ANNOUNCE_TABLEROOT, myname, tablerootname,
 				  tablerootaddr, tablerootport));
 #ifndef LOGINSERVER
-  if (strlen(globaldate) > 1)
+  if (strlen(globaldate) > 0)
     sendmessage(connstr,
 		makemsg2(ANNOUNCE_GLOBALDATE, globaldate, globalseed
 			 /*, seconds_to_next_week()*/
@@ -2418,8 +2427,8 @@ static void rejoin(void)
 #if DEBUGCONN
     DEBUG(talkmsg("Rejoining at loginserver"));
 #endif
-    strcpy(rejoinaddr, loginserveraddr);
-    strcpy(rejoinport, loginserverport);
+    strcpy(rejoinaddr, loginserveraddr());
+    strcpy(rejoinport, loginserverport());
     strcpy(rejoincap, TABLE_SERVERs);
     rejoinnumtries = NUMTRIES;
   }
@@ -2437,8 +2446,8 @@ static void rejoin(void)
 #if DEBUGCONN
     DEBUG(talkmsg("Will probably next try rejoining at loginserver"));
 #endif
-    strcpy(rejoinaddr, loginserveraddr);
-    strcpy(rejoinport, loginserverport);
+    strcpy(rejoinaddr, loginserveraddr());
+    strcpy(rejoinport, loginserverport());
     strcpy(rejoincap, TABLE_SERVERs);
     rejoinnumtries = NUMTRIES;
   }
@@ -2751,7 +2760,7 @@ static void handlemessage(message *m, char *msg)
       if (!tablehostmode) {
 	state = LIMBO;
 	if_not_connected_reset_tablehostname();
-	if (hasWindows) UIstate(state);
+	UIcommstate(state);
       }
     }
     else if (rho != NULL && streq(m->from, rho->name)) {
@@ -2776,7 +2785,7 @@ static void handlemessage(message *m, char *msg)
     case TABLE_SERVER:
       addwho(myname);
       state = CONNECTED;
-      if (hasWindows) UIstate(state);
+      UIcommstate(state);
 
       /* Copy info for possible later use if we are unexpectly disconnected;
 	 currently, however, this information is never used---we always try
@@ -2832,10 +2841,12 @@ static void handlemessage(message *m, char *msg)
 #endif
 #ifdef LOGINSERVER
       case TABLE_SERVER:
+#if 0  /* is it necessary to change this stuff here?! */
 	STRCPY(loginservername, myname);
 	STRCPY(loginserveraddr, localIPaddr);
 	STRCPY(loginserverport, localport);
 	/* status4(localIPaddr, ":", localport, " is loginserver location"); */
+#endif
 
 	if (version[0] != '8' && version[0] != '9') {
 	  sendmessage(connstr,
@@ -2874,7 +2885,7 @@ static void handlemessage(message *m, char *msg)
       sever_all_communication();
       state = LIMBO;
       if_not_connected_reset_tablehostname();
-      if (hasWindows) UIstate(state);
+      UIcommstate(state);
     }
     break;
     
@@ -2892,7 +2903,7 @@ static void handlemessage(message *m, char *msg)
     if (numargs != 1) goto wrongargs;
     addwho(argv[0]);
 #ifdef BEEPONJOIN
-  UIbeep(); /* this seems to be the spot for it -- Syela */
+    UIbeep();
 #endif
     if (tablehostmode) redescribetable();
     break;
@@ -2970,9 +2981,9 @@ static void handlemessage(message *m, char *msg)
     }
     
     argv[0] = TEMPCOPY(myname); /* use my name for broadcast to my children */
-    STRCPY(loginservername, argv[1]);
-    STRCPY(loginserveraddr, argv[2]);
-    STRCPY(loginserverport, argv[3]);
+    TclSet("loginservername", argv[1]);
+    TclSet("loginserveraddr", argv[2]);
+    TclSet("loginserverport", argv[3]);
     rebroadcast(m);
     break;
     
