@@ -19,6 +19,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *   Lex Spoon <lex@cc.gatech.edu>
  *
  * Alternatively, the contents of this file may be used under the
  * terms of either the GNU General Public License Version 2 or later
@@ -31,7 +32,16 @@
  * you do not delete the provisions above, a recipient may use your
  * version of this file under the terms of either the MPL or the GPL.
  * ***** END LICENSE BLOCK ***** */
+
+/* text.c - for both reading and writing to the screen when
+ * we don't have Tk
+ *
+ * $Id: text.c,v 1.9 2004/09/26 17:33:57 lexspoon Exp $
+ */
+
+
 #include "floater.h"
+#include "UI.h"
 
 #if GUI_ONLY||SILENT
 
@@ -44,10 +54,6 @@ fl_bool needrefresh = FALSE;
 #include "tickler.h"
 #include "text.h"
 
-/*
-  text.c: for both reading and writing to the screen when we don't have Tk
-*/
-
 
 extern fl_bool initializing; /* from main.c */
 
