@@ -32,7 +32,7 @@
  * version of this file under the terms of either the MPL or the GPL.
  * ***** END LICENSE BLOCK ***** */
 
-/* $Id: tickler.h,v 1.11 2004/06/27 18:56:38 lexspoon Exp $
+/* $Id: tickler.h,v 1.13 2005/08/20 20:30:33 lexspoon Exp $
  *
  * Interfacing with Tcl/Tk
  */
@@ -70,9 +70,6 @@ void setupbindings(void);
 	isin(c, "${}[]@#%^&*()`';:,.<>/?-_=+|~")))
 
 
-#define UIpatientcursor() TclDo("patientcursor")
-#define UInormalcursor() TclDo("normalcursor")
-
 void source(const char *s);
 char *TclDoIgnoreErrors(const char *s);
 char *TclDo(const char *s);
@@ -168,8 +165,8 @@ void talkm10(const char *, const char *, const char *, const char *,
  	     const char *, const char *, const char *, const char *,
 	     const char *, const char *);
 
-const char *tclclean(const char *);
-const char *braceclean(const char *);
+char *tclclean(const char *);
+char *braceclean(const char *);
 char *destructivebracketclean(char *s);
 #define fileclean(s) \
     destructiveBackslashToSlash(braceclean(destructivebracketclean(s)))
