@@ -37,23 +37,19 @@
 #           and keeping track of the open tables
 
 
-# XXX this doesn't work; autoconf only sets up directory substitutions for Makefiles
-#set lsfile @libdir@/serverfiles/loginserver.tcl
-set lsfile /var/lib/floater/serverfiles/loginserver.tcl
+set lsfile $server_dir/serverfiles/loginserver.tcl
 set writefile $lsfile\_wtest
 set lsdir [file dirname $lsfile]
-#set rsdir @rsdir@
-set rsdir /var/lib/floater/floater_rs
+set rsdir $server_dir/floater_rs
 
-#set webdir @webdir@
-set webdir /var/lib/floater/www
+set webdir $server_dir/www
 set rawresultsdir $webdir/rawresults
 set permdir $lsdir/perm
 set tmpdir $lsdir/tmp
 
-# 5 minutes
-set check_tempdir_interval [expr 1000 * 60 * 5]
-set flush_interval [expr 1000 * 60 * 5]
+# set these intervals to 5 minutes
+set check_tempdir_interval [expr 1000 * 60 * 5]  
+set flush_interval [expr 1000 * 60 * 5]     
 
 exec sh -c "cd $lsdir && make all || (echo make all failed | mail $bugmail)"
 
